/*
 * Jakarta Bean Validation API
 *
 * License: Apache License, Version 2.0
 * See the license.txt file in the root directory or <http://www.apache.org/licenses/LICENSE-2.0>.
 */
/**
 * Package related to the control and execution of executable validation.
 * Use {@link jakarta.validation.executable.ValidateOnExecution} to control whether or
 * not a (set of) constructors or methods are being validated.
 */
package jakarta.validation.executable;
