/*
 * Jakarta Bean Validation API
 *
 * License: Apache License, Version 2.0
 * See the license.txt file in the root directory or <http://www.apache.org/licenses/LICENSE-2.0>.
 */
/**
 * Contains all the Jakarta Bean Validation provided constraints
 * also called built-in constraints.
 * <p>
 * These constraints do not cover all functional use cases but do represent
 * all the fundamental blocks to express low level constraints on basic JDK
 * types.
 */
package jakarta.validation.constraints;
