/*
 * Jakarta Bean Validation API
 *
 * License: Apache License, Version 2.0
 * See the license.txt file in the root directory or <http://www.apache.org/licenses/LICENSE-2.0>.
 */
/**
 * Package containing constructs specific to constraint validators.
 * In particular {@link jakarta.validation.constraintvalidation.SupportedValidationTarget}
 * is used to refine the validation target of a {@link jakarta.validation.ConstraintValidator}.
 */
package jakarta.validation.constraintvalidation;
