/*
 * Jakarta Bean Validation API
 *
 * License: Apache License, Version 2.0
 * See the license.txt file in the root directory or <http://www.apache.org/licenses/LICENSE-2.0>.
 */
/**
 * Package containing objects specific to the interaction between the
 * Jakarta Bean Validation bootstrapping process and Jakarta Bean Validation providers.
 */
package jakarta.validation.spi;
